@@ -21,8 +21,6 @@
 
 #include "gl_xlist.h"
 
-#include "xalloc_extra.h"
-
 /* basic.c ---------------------------------------------------------------- */
 size_t get_goalc_bp (Buffer * bp, Point pt);
 size_t get_goalc (void);
@@ -163,9 +161,6 @@ astr chordtostr (size_t key);
 gl_list_t keystrtovec (const char *key);
 astr keyvectostr (gl_list_t keys);
 
-/* killring.c ------------------------------------------------------------- */
-void free_kill_ring (void);
-
 /* line.c ----------------------------------------------------------------- */
 #define FIELD(ty, field)                                \
   ty get_line_ ## field (const Line *wp);               \
@@ -178,7 +173,7 @@ void line_delete (Line *l);
 Line *line_insert (Line *l, astr i);
 void line_remove (Line *l);
 void line_replace_text (Line * lp, size_t offset, size_t oldlen,
-                        char *newtext, int replace_case);
+                        const char *newtext, int replace_case);
 int insert_char (int c);
 int insert_char_in_insert_mode (int c);
 bool fill_break_line (void);
