@@ -1,3 +1,3 @@
 (insert-file "insert_file.input")
 (save-buffer)
-(save-buffers-kill-zile)
+(save-buffers-kill-emacs)
